/*
Copyright © 2019-2020 befovy <befovy@gmail.com>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
package fvmgo

import (
  "fmt"
  "io"
  "io/ioutil"
  "os"
  "path/filepath"
)

// IsFileExists checks if a file exists and is not a directory
func IsFileExists(name string) bool {
  info, err := os.Stat(name)
  if err != nil {
    return false
  }
  return !info.IsDir()
}

// IsDirectory check if path exists and is a directory
func IsDirectory(name string) bool {
  info, err := os.Stat(name)
  if err != nil {
    return false
  }
  return info.IsDir()
}

func IsEmptyDir(name string) (bool, error) {
  entries, err := ioutil.ReadDir(name)
  if err != nil {
    return false, err
  }
  return len(entries) == 0, nil
}

func IsSymlink(name string) bool {
  info, err := os.Lstat(name)
  if os.IsNotExist(err) {
    return false
  } else if err != nil {
    Warnf("Error when check symlink: %v", err)
    return false
  }
  return (info.Mode() & os.ModeSymlink) != 0
}

func IsNotFound(name string) bool {
  _, err := os.Lstat(name)
  return err != nil
}

func IsExecutable(name string) bool {
  info, err := os.Stat(name)
  if err != nil {
    return false
  }
  return !info.IsDir() && ((info.Mode() & 0111) != 0)
}

// CopyFile copies the contents of the file named src to the file named
// by dst. The file will be created if it does not already exist. If the
// destination file exists, all it's contents will be replaced by the contents
// of the source file. The file mode will be copied from the source and
// the copied data is synced/flushed to stable storage.
func CopyFile(src, dst string) (err error) {
  in, err := os.Open(src)
  if err != nil {
    return
  }
  defer in.Close()

  out, err := os.Create(dst)
  if err != nil {
    return
  }
  defer func() {
    if e := out.Close(); e != nil {
      err = e
    }
  }()

  _, err = io.Copy(out, in)
  if err != nil {
    return
  }

  err = out.Sync()
  if err != nil {
    return
  }

  si, err := os.Stat(src)
  if err != nil {
    return
  }
  err = os.Chmod(dst, si.Mode())
  if err != nil {
    return
  }

  return
}

// CopyDir recursively copies a directory tree, attempting to preserve permissions.
// Source directory must exist, destination directory must *not* exist.
// Symlinks are ignored and skipped.
func CopyDir(src string, dst string) (err error) {
  src = filepath.Clean(src)
  dst = filepath.Clean(dst)

  si, err := os.Stat(src)
  if err != nil {
    return err
  }
  if !si.IsDir() {
    return fmt.Errorf("source is not a directory")
  }

  _, err = os.Stat(dst)
  if err != nil && !os.IsNotExist(err) {
    return
  }
  if err == nil {
    return fmt.Errorf("destination already exists")
  }

  err = os.MkdirAll(dst, si.Mode())
  if err != nil {
    return
  }

  entries, err := ioutil.ReadDir(src)
  if err != nil {
    return
  }

  for _, entry := range entries {
    srcPath := filepath.Join(src, entry.Name())
    dstPath := filepath.Join(dst, entry.Name())

    if entry.IsDir() {
      err = CopyDir(srcPath, dstPath)
      if err != nil {
        return
      }
    } else {
      // Skip symlinks.
      if entry.Mode()&os.ModeSymlink != 0 {
        continue
      }

      err = CopyFile(srcPath, dstPath)
      if err != nil {
        return
      }
    }
  }

  return
}
